package net.pingex.discordbot;

import sx.blah.discord.api.ClientBuilder;
import sx.blah.discord.api.Event;
import sx.blah.discord.api.IDiscordClient;
import sx.blah.discord.api.IListener;
import sx.blah.discord.handle.impl.events.ReadyEvent;
import sx.blah.discord.util.DiscordException;
import java.io.IOException;
import java.util.logging.Logger;

/**
 * Main Entry Point
 * @version 0.1-dev
 * @author Raphael "Pingex" NAAS
 */
public class DiscordBot
{
	private static final Logger LOGGER = Logger.getLogger(DiscordBot.class.getName());
	private IDiscordClient client;

	public DiscordBot(IDiscordClient client)
	{
		this.client = client;
		LOGGER.info("Successfully logged in !");

		this.client.getDispatcher().registerListener((IListener<ReadyEvent>) event ->
				LOGGER.info("Logged in as \"" + event.getClient().getOurUser().getName() + "#" + event.getClient().getOurUser().getDiscriminator() + "\" (#" + event.getClient().getOurUser().getID() + ")"));
	}

	// ====================================================
	// Static functions: main, config loader, etc.
	// ====================================================

	/**
	 * Entry point
	 * @param args Arguments passed to java
	 */
	public static void main(String[] args)
	{
		LOGGER.info("Hello World, starting up");

		// LOAD CONFIGURATION
		try
		{
			Configuration.loadConfiguration("config.ini");
		} catch (IOException e)
		{
			LOGGER.severe("Could not load Configuration, reason: " + e.getMessage());
			System.exit(10);
		}

		LOGGER.info("Logging in to Discord");
		try
		{
			loginToDiscord();
		} catch (ConfigurationException e)
		{
			LOGGER.severe("There is a problem with the Configuration: " + e.getMessage());
			System.exit(9);
		} catch (DiscordException e)
		{
			LOGGER.severe(e.getMessage());
			System.exit(2);
		}
	}

	private static DiscordBot loginToDiscord() throws ConfigurationException, DiscordException
	{
		ClientBuilder builder = new ClientBuilder();
		if(Configuration.getValue("discord", "isBot", boolean.class))
		{
			if(!Configuration.exists("discord", "token"))
				throw new ConfigurationException("No login token defined");
			builder.withToken(Configuration.getValue("discord", "token"));
		}
		else
		{
			if(!Configuration.exists("discord", "email") || !Configuration.exists("discord", "password"))
				throw new ConfigurationException("Can't login: Either the email or the password wasn't provided.");
			builder.withLogin(Configuration.getValue("discord", "email"), Configuration.getValue("discord", "password"));
		}

		return new DiscordBot(builder.login());
	}
}
